<?php
/**
 * Add Genericons
 *
 * @link https://genericons.com/
 *
 * @package Suri
 * @since 0.1.2
 */

/**
 * Add support for Genericons.
 *
 * @since 0.1.2
 */
class Suri_Genericons {

	/**
	 * Holds the instance of this class.
	 *
	 * @since  0.1.8
	 * @access protected
	 * @var    object
	 */
	protected static $instance = null;

	/**
	 * Hold element's data for which fonticons to be generated.
	 *
	 * @since  0.1.8
	 * @access public
	 * @var    array
	 */
	public $element_fonticons;

	/**
	 * Hold social fonticons css classes.
	 *
	 * @since  0.1.8
	 * @access public
	 * @var    array
	 */
	public $social_fonticons;

	/**
	 * Constructor method.
	 *
	 * @since 0.1.8
	 */
	public function __construct() {
		$this->element_fonticons = suri_icon_fonts_array( 'all' );
		$this->social_fonticons  = suri_social_icons_array( 'all' );
	}

	/**
	 * Register hooked functions.
	 *
	 * @since 0.1.8
	 */
	public static function init() {
		add_action( 'wp_enqueue_scripts' , array( Suri_Genericons::get_instance(), 'enqueue_genericons' ) );
		add_filter( 'nav_menu_link_attributes' , array( Suri_Genericons::get_instance(), 'menu_font_class' ), 10, 4 );
		add_filter( 'suri_theme_controls', array( Suri_Genericons::get_instance(), 'customizer_controls' ) );
		add_filter( 'suri_theme_defaults', array( Suri_Genericons::get_instance(), 'theme_defaults' ) );
		foreach ( Suri_Genericons::get_instance()->element_fonticons as $element => $value ) {
			add_filter( "suri_get_attr_{$element}", array( Suri_Genericons::get_instance(), 'font_class' ), 10, 2 );
		}
	}

	/**
	 * Enqueue genericons stylesheet.
	 *
	 * @since 0.1.2
	 */
	public function enqueue_genericons() {
		// Enqueue font icons stylesheet.
		if ( 1 === get_theme_mod( 'suri_use_full_genericons', suri_get_theme_defaults( 'suri_use_full_genericons' ) ) ) :
			wp_enqueue_style( 'suri-genericons', get_template_directory_uri() . '/resources/fonts/genericons/genericons.css', array(), '3.4.1' );
		elseif ( has_nav_menu( 'social' ) ) :
			wp_enqueue_style( 'suri-genericons', get_template_directory_uri() . '/resources/fonts/genericons/gensocial.css', array(), '1.0.0' );
		else :
			wp_enqueue_style( 'suri-genericons', get_template_directory_uri() . '/resources/fonts/genericons/genbasic.css', array(), '1.0.0' );
		endif;
	}

	/**
	 * Set genericons customizer controls and settings.
	 *
	 * @since 0.2.2
	 *
	 * @param array $control Array of default theme controls and settings.
	 * @return array Returns array of default theme controls and settings.
	 */
	public function customizer_controls( $control ) {
		$control[] = array(
			'label'       => esc_html__( 'Load all genericon font glyphs', 'suri' ),
			'section'     => 'suri_content_section',
			'settings'    => 'suri_use_full_genericons',
			'description' => esc_html__( 'By default only few font glyphs will be loaded.', 'suri' ),
			'type'        => 'checkbox',
		);
		return $control;
	}

	/**
	 * Default values for genericons customization option.
	 *
	 * @since 0.2.2
	 *
	 * @param mixed $defaults integer, string or array option values.
	 * @return mixed Returns integer, string or array option values.
	 */
	public function theme_defaults( $defaults ) {
		$defaults['suri_use_full_genericons'] = '';
		return $defaults;
	}

	/**
	 * Adds appropriate css classes to elements.
	 *
	 * @since 0.1.8
	 *
	 * @param  array $attr Array of attributes to pass in (overwrites filters).
	 * @param  str   $slug The slug/ID of the element (e.g., 'sidebar').
	 */
	public function font_class( $attr, $slug ) {
		$element_options = $this->element_fonticons[ $slug ];
		if ( 'before' === $element_options['location'] ) :
			$class = 'suri-gen-b ';
		else :
			$class = 'suri-gen-a ';
		endif;
		$class .= 'sf-' . $element_options['class'] . '-sf';
		$attr['class'] .= sprintf( ' %s', $class );
		return $attr;
	}

	/**
	 * Adds css classes to social menu links.
	 *
	 * Adds appropriate css classes to social menu links based on
	 * link URL.
	 *
	 * @since 0.1.8
	 *
	 * @param array $atts HTML attributes applied to the menu item's <a> element.
	 * @param obj   $item The current menu item object.
	 * @param array $args An array of wp_nav_menu() arguments.
	 * @param int   $depth Depth of menu item. Used for padding.
	 */
	public function menu_font_class( $atts, $item, $args, $depth ) {
		$classes = empty( $item->classes ) ? array() : (array) $item->classes;

		// Return navigation menu arrow icon class.
		if ( ( 'social' !== $args->theme_location ) && ( 'footer' !== $args->theme_location ) ) :
			if ( in_array( 'menu-item-has-children', $classes, true ) ) :
				$atts['class'] = 'menu-link suri-gen-a sf-genericon genericon-expand-sf';
			endif;
			return $atts;
		endif;

		// Return social menu icon class.
		if ( 'social' === $args->theme_location ) :
			$icon_class = $this->social_fonticons['default'];
			foreach ( $this->social_fonticons as $name => $classes ) {
				if ( stripos( $atts['href'], $name ) !== false ) :
					$icon_class = $classes;
					break;
				endif;
			}

			$atts['class'] = 'social-link suri-gen-b sf-' . esc_attr( $icon_class ) . '-sf';
			return $atts;
		endif;

		return $atts;
	}

	/**
	 * Returns the instance.
	 *
	 * @since  0.1.8
	 *
	 * @return object Customizer instance.
	 */
	public static function get_instance() {
		null === self::$instance && self::$instance = new self;
		return self::$instance;
	}
}

Suri_Genericons::init();
